#!/usr/bin/env python

import click

from kudu.api import authenticate
from kudu.commands.init import init
from kudu.commands.link import link
from kudu.commands.pull import pull
from kudu.commands.push import push
from kudu.commands.create import create
from kudu.config import ConfigOption


@click.group()
@click.option(
    '--username', '-u', cls=ConfigOption, prompt=True, envvar='KUDU_USERNAME'
)
@click.option(
    '--password',
    '-p',
    cls=ConfigOption,
    prompt=True,
    hide_input=True,
    envvar='KUDU_PASSWORD'
)
@click.option('--token', '-t', cls=ConfigOption, envvar='KUDU_TOKEN')
@click.pass_context
def cli(ctx, username, password, token):
    if not token:
        try:
            token = authenticate(username, password)
        except ValueError:
            click.echo('Invalid username or password', err=True)
            exit(1)

    ctx.obj = {'username': username, 'password': password, 'token': token}


cli.add_command(init)
cli.add_command(pull)
cli.add_command(push)
cli.add_command(link)
cli.add_command(create)

if __name__ == '__main__':
    cli()



import os
import time
import click
import requests

from kudu.api import request
from kudu.commands.push import get_file_data, update_file_metadata

@click.command()
@click.option('--instance', '-i', type=int, required=True, help="instance id to upload file")
@click.option('--body', '-b', type=str, required=True, help="Body of the file")
@click.option('--filename', '-f', type=str, required=False, help="Name of the file in bucket")
@click.option('--path', '-p', type=click.Path(exists=True), default=None)
@click.option('--extension', '-e', type=str, required=False, default="zip", help="Extension of the file that's going to be uploaded, default 'zip'")
@click.pass_context
def create(ctx, instance, body, filename = None, path = None, extension = "zip"):
    file_data = get_file_data(path, filename or '%i%s' % (int(round(time.time() * 1000)), extension), category=extension)
    file_id = create_file(ctx.obj['token'], instance, body, extension, filename=filename)
    url = '/files/%d/upload-url/' % file_id
    response = request('get', url, token=ctx.obj['token'])
    # upload data
    requests.put(response.json(), data=file_data)

    # touch file
    update_file_metadata(ctx, file_id)


def create_file(token, app_id, file_body, category, filename = None):
    payload = {
        'app':
            app_id,
        'body':
            file_body,
        'downloadUrl':
            'https://admin.pitcher.com/downloads/Pitcher%20HTML5%20Folder.zip',
        'category': category,
        'dont_convert': True
    }

    if filename:
        payload['filename'] = filename
    
    res = request(
        'post',
        '/files/',
        json=payload,
        token=token
    )
    json = res.json()

    if res.status_code != 201 and res.status_code != 200:
        if json.get('app'):
            click.echo('Invalid instance', err=True)
        else:
            click.echo('Unknown error', err=True)
        print(json)
        exit(1)

    return json.get('id')



import os
from collections import namedtuple
from datetime import datetime

import click
import requests

from kudu.api import request as api_request
from kudu.config import ConfigOption
from kudu.mkztemp import NameRule, mkztemp
from kudu.types import PitcherFileType

CategoryRule = namedtuple('crule', ('category', 'rule'))


CATEGORY_RULES = (
    CategoryRule('',
                 NameRule((r'^interface', r'(.+)'), ('{base_name}', '{0}'))),
    CategoryRule(('presentation', 'zip'),
                 NameRule(r'^thumbnail.png', '{base_name}.png')),
    CategoryRule(('presentation', 'zip'),
                 NameRule(r'(.+)', ('{base_name}', '{0}'))),
) # yapf: disable


@click.command()
@click.option(
    '--file',
    '-f',
    'pf',
    cls=ConfigOption,
    config_name='file_id',
    prompt=True,
    type=PitcherFileType(category=('zip', 'presentation', 'json', ''))
)
@click.option('--path', '-p', type=click.Path(exists=True), default=None)
@click.pass_context
def push(ctx, pf, path):
    name = pf['filename']
    base_name, _ = os.path.splitext(name)

    url = '/files/%d/upload-url/' % pf['id']
    response = api_request('get', url, token=ctx.obj['token'])

    data = get_file_data(path, base_name, pf['category'])

    # upload data
    requests.put(response.json(), data=data)
    
    # touch file
    update_file_metadata(ctx, pf['id'])

def get_file_data(path, base_name, category):
    if path is None or os.path.isdir(path):
        rules = [c.rule for c in CATEGORY_RULES if category in c.category]
        fp, _ = mkztemp(base_name, root_dir=path, name_rules=rules)
        data = os.fdopen(fp, 'r+b')
    else:
        data = open(path, 'r+b')

    return data


def update_file_metadata(ctx, file_id):
    # touch file
    url = '/files/%d/' % file_id
    json = {
        'creationTime': datetime.utcnow().isoformat(),
        'metadata': get_metadata_with_github_info(ctx, file_id)
    }
    api_request('patch', url, json=json, token=ctx.obj['token'])

def get_metadata_with_github_info(ctx, file_id):
    # first get existing metadata then modify it
    url = '/files/%d/' % file_id
    response = api_request('get', url, token=ctx.obj['token']).json()
    metadata = response.get('metadata', {})

    # NOT losing repo info for non-github deployments
    current_repo_info = metadata.get('GITHUB_REPOSITORY', 'not_available') 
    metadata['GITHUB_REPOSITORY'] = os.environ.get('GITHUB_REPOSITORY', current_repo_info)

    # losing commit SHA and run id info for non-github deployments
    metadata['GITHUB_SHA'] = os.environ.get('GITHUB_SHA', 'not_available')
    metadata['GITHUB_RUN_ID'] = os.environ.get('GITHUB_RUN_ID', 'not_available')

    return metadata